%% function that calculates the optimal scheme for provision of ancillary services
% Gradient Search Algorithm

function [SOC_adapt, Bid, P_upward, P_downward, ConsumE, fval_up, fval_down] = ...
            Adapt_optimal_GSA (EV_n, EV_P, EV_E, EV_batt, subrange, range_1, ...
            range_2, Penet_local,usage_local,available_local,charging,SOC_ini,day,conv_topo,scheme)

Avai_P_EV = available_local*EV_n*EV_P/1000; % MW - available power to be absorved by EV at every hour of the day
ConsumE = (usage_local * EV_n* EV_E)/1000; % MWh - Energy consumed by fleet of EVs        
Max_SOC = (EV_batt * EV_n)/1000; % MWh - Maximum SOC allow in the fleet
for i = 1:6
    Media(i) = mean(Penet_local(i*4-3:i*4)); % Average value of penetration in the block
    ConsumE_blk(i) = sum(ConsumE(i*4-3:i*4)); % Energy consumed by block
    min_Avai_P (i) = min(Avai_P_EV(i*4-3:i*4)); % minimum power available each block
end
f_AS_up = [0.004 0.006 0.007 0.008 0.021 0.04]; % Percentile 20th
f_AS_down = [0.049 0.041 0.040 0.035 0.059 0.112]; % Percentile 80th    

% fmincon options
options = optimset('LargeScale','off');
options = optimset(options,'DiffMinChange',0.01);
options = optimset(options,'DiffMaxChange',0.1);
options = optimset(options, 'algorithm','interior-point');
options = optimset(options, 'TolX', 1e-6);
options = optimset(options, 'MaxIter',100);

%% Optimization function for upward provision of frequency regulation
% Initialize variables
X0 = ones(6,1)*0.3; % needs to be initialize at the lower boundary

% Boundaries
lb = zeros(length(X0),1); % Power must be positive
ub = min_Avai_P; % Power must be smaller than available

% Constraints
    A_vec = ones(length(X0),1)*-1;
    A = diag(A_vec);
    b = ones(6,1)*-0.3; 
    
for i = 1:6
    if Media(i) < 0.5 & SOC_ini > 0.6 * Max_SOC
        if conv_topo == 1
            b(i) = 0;
        end
        ub(i) = 0;
        X0(i) = 0;
    end
end
    
if SOC_ini > 0.9*Max_SOC
    SOC_ini = 0.9*Max_SOC
    X0(1) = 0;
end

if conv_topo == 1 % pre-selection of variables. Only for uni-directional
SOC_max = [0.9 0.895 0.89 0.885 0.88 0.875] * Max_SOC;
out_exc = 1;
SOC(1) = SOC_ini;
while out_exc == 1
    out_exc = 0;
    SOC(2:7) = zeros(1,6);
for j = 1:6
    SOC(j+1) = SOC(j) + 4*X0(j)*(1-f_AS_up(j)) - ConsumE_blk(j);
    if SOC(j+1) > SOC_max (j)
        out_exc = 1;
        Media_select = ones(j,1)*5;
        for s=1:j
            if b(s) ~= 0 % in case the block has already been forced to zero
                Media_select(s) = Media(s);
            end
        end
        [min_val, min_index] = min(Media_select(1:j)); % identify minimum penetration value and index
        blk_curtail = ceil(min_index); % identify block of minimum penetration
    
        b(blk_curtail) = 0; % force optimal solution of the withdrawed block to zero
        ub(blk_curtail) = 0; 
        X0(blk_curtail) = 0;
        break
    end
    
end
end
end

if conv_topo == 2 % Uni-directional capability does not need these constraints
    A = [];
    b = [];
end

    save variables_up.mat ConsumE_blk Max_SOC SOC_ini f_AS_up Media min_Avai_P conv_topo scheme
    fprintf('Day %.0f, calculating Upward regulation \n Active variables \t',day)
    disp(X0')

    [x,fval_up,exitflag,output] = fmincon(@myfun_up_GSA,X0,A,b,[],[],lb,ub,@mycon_up_GSA, options)

% Constraint verification
if output.constrviolation > 1e-3 % if constraints are violated, the block with the lowest penetration is withdrawed from the variables
    disp('function does not constraint upward----------------------------------')
end  

% Outputs
if conv_topo == 1
    P_upward = x'; % It is the Bid for day-ahead market and also bid for power in upward regulation
else
    P_upward = x' + min_Avai_P;
end

P_pre_down = min_Avai_P' - x;
for i = 1:6
    Bid(i*4-3:i*4) = repmat(x(i),1,4);
    if P_pre_down(i) < 0.3 
        P_pre_down(i) = 0;
    end
end

SOC_adapt (1) = SOC_ini;
for i=1:6
    SOC_adapt(i+1) = SOC_adapt(i) + 4*x(i)...
        - 4*P_upward(i)*f_AS_up(i) - ConsumE_blk(i);
end

%% Optimization function for downward provision of frequency regulation
% Initialize variables
X0 = ones(6,1)*0.3;

% Constraints
A_vec = ones(length(X0),1)*-1;
A = diag(A_vec);
b = ones(length(X0),1)*-0.3; 

% Boundaries
lb = zeros(length(X0),1); % Power must be positive
ub = P_pre_down; 

for i = 1:6
    if P_pre_down (i) == 0
        b(i) = 0;
        ub (i) = 0;
        X0 (i) = 0;
    end
end
 
% Restrict number of variables
out_exc = 1;
while out_exc == 1
    out_exc = 0;
    SOC_down = [SOC_adapt(1) zeros(1,6)];
for i = 1:6
    SOC_down(i+1) = SOC_adapt(i+1) + sum(4*X0(1:i).*f_AS_down(1:i)');
    if SOC_down(i+1) > 0.9*Max_SOC
        out_exc = 1;
        Media_select = ones(i,1)*5;
        for s=1:i
            if b(s) ~= 0 % in case the block has already been forced to zero
                Media_select(s) = Media(s);
            end
        end
        [min_val, min_index] = min(Media_select(1:i)); % identify minimum penetration value and index
        blk_curtail = ceil(min_index); % identify block of minimum penetration
    
        b(blk_curtail) = 0; % force optimal solution of the withdrawed block to zero
        ub(blk_curtail) = 0; 
        X0(blk_curtail) = 0;
        break
    end
    
end
end


    save variables_down.mat Max_SOC SOC_adapt f_AS_down
    fprintf('Day %.0f, calculating Downward regulation \n Active variables \t',day)
    disp(X0')    
    
[x,fval_down,exitflag,output] = ...
    fmincon(@myfun_down_GSA,X0,A,b,[],[],lb,ub,@mycon_down_GSA, options)

% Constraint verification 
if output.constrviolation > 1e-3 % if constraints are violated, the block with the lowest penetration is withdrawed from the variables
    disp('function does not constraint downward----------------------------------')
end    

% Outputs
P_downward = x';
SOC_adapt (1) = SOC_ini;
for i=1:24
    SOC_adapt(i+1) = SOC_adapt(i) + Bid(i) + P_downward(ceil(i/4))*f_AS_down(ceil(i/4))...
        - P_upward(ceil(i/4))*f_AS_up(ceil(i/4)) - ConsumE(i) ;
end